@@ -9,6 +9,7 @@ import smtplib
 from xml.dom.minidom import Document
 
 from django.conf import settings
+from django.contrib.sites.models import Site
 from django.core.exceptions import FieldError
 from django.core.mail import EmailMultiAlternatives
 from django.core.paginator import Paginator, EmptyPage, InvalidPage
@@ -17,6 +18,7 @@ from django.db.models import Q
 from django.http import HttpResponse, HttpResponseRedirect, Http404
 from django.shortcuts import get_object_or_404, render_to_response
 from django.template import RequestContext as _rc
+from django.template.loader import render_to_string
 from django.utils.html import escape
 
 from .forms import forms_for_survey
@@ -37,10 +39,67 @@ from .models import (
 from .jsonutils import dump, dumps, datetime_to_string
 
 from .util import ChoiceEnum, get_function
-from . import settings as local_settings
+from . import settings as crowdsourcing_settings
+
+
+def allow_origin_sites():
+    database_sites = []
+    sites = Site.objects.all()
+    for protocol in ["http", "https"]:
+        database_sites += ["%s://%s" % (protocol, s.domain) for s in sites]
+    config_sites = getattr(settings, "ADDITIONAL_CORS_SITES", [])
+    return list(set(database_sites + config_sites))
+
+
+def api_response(request, data, callback=None, format='json'):
+    # http://www.loggly.com/blog/2011/12/enabling-cors-in-django-piston/
+    # for how to enable CORS
+    request_method = request.method.upper()
+    if request_method in ["OPTIONS", "HEAD"]:
+        response = HttpResponse()
+    elif isinstance(data, HttpResponseRedirect):
+        response = data
+    elif callback:
+        body = u'%s(%s);' % (callback, dumps(data))
+        response = HttpResponse(body, mimetype='application/javascript')
+    elif format == 'html':
+        response = HttpResponse(data)
+    else:
+        response = HttpResponse(mimetype='application/json')
+        dump(data, response)
+
+    origin = request.META.get('HTTP_REFERER', '') or \
+             request.META.get('HTTP_ORIGIN', '')
+    allowed = allow_origin_sites()
+    if origin:
+        allowed = [a for a in allowed if origin.find(a) >= 0]
+    response["Access-Control-Allow-Origin"] = " ".join(allowed)
+    response['Access-Control-Allow-Methods'] = \
+        'POST, GET, OPTIONS, HEAD, PUT, DELETE'
+    response["Access-Control-Allow-Headers"] = 'Authorization'
+    response["Access-Control-Allow-Credentials"] = 'true'
+
+    return response
+
+
+def api_response_decorator(format='json'):
+    def _api_response_decorator(the_func):
+        def _decorated(request, *args, **kwargs):
+            request_method = request.method.upper()
+            if request_method == "OPTIONS":
+                return api_response(request, {}, format=format)
+            result = the_func(request, *args, **kwargs)
+            if isinstance(result, HttpResponse):
+                return result
+            callback = request.GET.get('callback', None)
+            return api_response(request, result, callback=callback)
+        return _decorated
+    return _api_response_decorator
 
 
 def _user_entered_survey(request, survey):
+    if not request.user.is_authenticated():
+        return False
     return bool(survey.submissions_for(
         request.user,
         request.session.session_key.lower()).count())
@@ -61,8 +120,9 @@ def _get_remote_ip(request):
 
 
 def _login_url(request):
-    if local_settings.LOGIN_VIEW:
-        return reverse(local_settings.LOGIN_VIEW) + '?next=%s' % request.path
+    if crowdsourcing_settings.LOGIN_VIEW:
+        start_with = reverse(crowdsourcing_settings.LOGIN_VIEW) + '?next=%s'
+        return start_with % request.path
     return "/?login_required=true"
 
 
@@ -131,7 +191,7 @@ def _url_for_edit(request, obj):
         # Probably 'admin' is not a registered namespace on a site without an
         # admin. Just fake it.
         edit_url = "/admin/%s/%s/%d/" % (view_args + (obj.id,))
-    admin_url = local_settings.SURVEY_ADMIN_SITE
+    admin_url = crowdsourcing_settings.SURVEY_ADMIN_SITE
     if not admin_url:
         admin_url = "http://" + request.META["HTTP_HOST"]
     elif len(admin_url) < 4 or admin_url[:4].lower() != "http":
@@ -141,7 +201,7 @@ def _url_for_edit(request, obj):
 
 def _send_survey_email(request, survey, submission):
     subject = survey.title
-    sender = local_settings.SURVEY_EMAIL_FROM
+    sender = crowdsourcing_settings.SURVEY_EMAIL_FROM
     links = [(_url_for_edit(request, submission), "Edit Submission"),
              (_url_for_edit(request, survey), "Edit Survey"),]
     if survey.can_have_public_submissions():
@@ -210,6 +270,7 @@ def survey_detail(request, slug):
     return _survey_show_form(request, survey, forms)
 
 
+@api_response_decorator(format='html')
 def embeded_survey_questions(request, slug):
     survey = _get_survey_or_404(slug, request)
     templates = ['crowdsourcing/embeded_survey_questions_%s.html' % slug,
@@ -220,7 +281,7 @@ def embeded_survey_questions(request, slug):
         if request.method == 'POST':
             if _submit_valid_forms(forms, request, survey):
                 forms = ()
-    return render_to_response(templates, dict(
+    return render_to_string(templates, dict(
         entered=_user_entered_survey(request, survey),
         request=request,
         forms=forms,
@@ -240,21 +301,20 @@ def _survey_report_url(survey):
                    kwargs={'slug': survey.slug})
 
 
+@api_response_decorator()
 def allowed_actions(request, slug):
     survey = _get_survey_or_404(slug, request)
     authenticated = request.user.is_authenticated()
-    response = HttpResponse(mimetype='application/json')
-    dump({"enter": _can_show_form(request, survey),
-          "view": survey.can_have_public_submissions(),
-          "open": survey.is_open,
-          "need_login": survey.require_login and not authenticated}, response)
-    return response
+    return {
+        "enter": _can_show_form(request, survey),
+        "view": survey.can_have_public_submissions(),
+        "open": survey.is_open,
+        "need_login": survey.require_login and not authenticated}
 
 
+@api_response_decorator()
 def questions(request, slug):
-    response = HttpResponse(mimetype='application/json')
-    dump(_get_survey_or_404(slug, request).to_jsondata(), response)
-    return response
+    return _get_survey_or_404(slug, request).to_jsondata()
 
 
 def submissions(request, format):
@@ -418,7 +478,6 @@ def submissions(request, format):
                     cell.appendChild(doc.createTextNode(u"%s" % value))
         response = HttpResponse(doc.toxml(), mimetype='text/xml')
     elif format == 'html': # mostly for debugging.
-        data_list = []
         keys = get_keys()
         results = [
             "<html><body><table>",
@@ -486,9 +545,13 @@ def _default_report(survey):
 def survey_report(request, slug, report='', page=None):
     templates = ['crowdsourcing/survey_report_%s.html' % slug,
                  'crowdsourcing/survey_report.html']
-    return _survey_report(request, slug, report, page, templates)
+    result = _survey_report(request, slug, report, page, templates)
+    if isinstance(result, HttpResponse):
+        return result
+    return HttpResponse(result)
 
 
+@api_response_decorator(format='html')
 def embeded_survey_report(request, slug, report=''):
     templates = ['crowdsourcing/embeded_survey_report_%s.html' % slug,
                  'crowdsourcing/embeded_survey_report.html']
@@ -498,7 +561,7 @@ def embeded_survey_report(request, slug, report=''):
 def _survey_report(request, slug, report, page, templates):
     """ Show a report for the survey. As rating is done in a separate
     application we don't directly check request.GET["sort"] here.
-    local_settings.PRE_REPORT is the place for that. """
+    crowdsourcing_settings.PRE_REPORT is the place for that. """
     if page is None:
         page = 1
     else:
@@ -516,28 +579,32 @@ def _survey_report(request, slug, report, page, templates):
         report_obj = get_object_or_404(reports, slug=report)
     elif survey.default_report:
         args = {"slug": survey.slug, "report": survey.default_report.slug}
-        return HttpResponseRedirect(reverse("survey_report_page_1",
-                                    kwargs=args))
+        url = reverse("survey_report_page_1", kwargs=args)
+        return HttpResponseRedirect(url)
     else:
         report_obj = _default_report(survey)
 
     archive_fields = list(survey.get_public_archive_fields())
     is_staff = request.user.is_staff
     if is_staff:
+        submissions = survey.submission_set.all()
         fields = list(survey.get_fields())
     else:
+        submissions = survey.public_submissions()
         fields = list(survey.get_public_fields())
     filters = get_filters(survey, request.GET)
 
-    public = survey.public_submissions()
     id_field = "crowdsourcing_submission.id"
     if not report_obj.display_individual_results:
-        submissions = public.none()
+        submissions = submissions.none()
     else:
-        submissions = extra_from_filters(public, id_field, survey, request.GET)
+        submissions = extra_from_filters(submissions,
+                                         id_field,
+                                         survey,
+                                         request.GET)
         # If you want to sort based on rating, wire it up here.
-        if local_settings.PRE_REPORT:
-            pre_report = get_function(local_settings.PRE_REPORT)
+        if crowdsourcing_settings.PRE_REPORT:
+            pre_report = get_function(crowdsourcing_settings.PRE_REPORT)
             submissions = pre_report(
                 submissions=submissions,
                 report=report_obj,
@@ -552,14 +619,7 @@ def _survey_report(request, slug, report, page, templates):
         page_obj.object_list,
         include_private_questions=is_staff)
 
-    pages_to_link = []
-    for i in range(page - 5, page + 5):
-        if 1 <= i <= paginator.num_pages:
-            pages_to_link.append(i)
-    if pages_to_link[0] > 1:
-        pages_to_link = [1, False] + pages_to_link
-    if pages_to_link[-1] < paginator.num_pages:
-        pages_to_link = pages_to_link + [False, paginator.num_pages]
+    pages_to_link = pages_to_link_from_paginator(page, paginator)
 
     display_individual_results = all([
         report_obj.display_individual_results,
@@ -579,7 +639,32 @@ def _survey_report(request, slug, report, page, templates):
         display_individual_results=display_individual_results,
         request=request)
 
-    return render_to_response(templates, context, _rc(request))
+    return render_to_string(templates, context, _rc(request))
+
+
+def pages_to_link_from_paginator(page, paginator):
+    """ Return an array with numbers where you should link to a page, and False
+    where you should show elipses. For example, if you have 9 pages and you are
+    on page 9, return [1, False, 5, 6, 7, 8, 9]. """
+    pages = []
+    for i in range(page - 4, page + 5):
+        if 1 <= i <= paginator.num_pages:
+            pages.append(i)
+    if pages[0] > 1:
+        pages = [1, False] + pages
+    if pages[-1] < paginator.num_pages:
+        pages = pages + [False, paginator.num_pages]
+
+    DISCARD = -999
+    for i in range(1, len(pages) - 1):
+        if pages[i - 1] + 2 == pages[i + 1]:
+            # Turn [1, False, 3... into [1, 2, 3
+            pages[i] = (pages[i - 1] + pages[i + 1]) / 2
+        elif pages[i - 1] + 1 == pages[i + 1]:
+            # Turn [1, False, 2... into [1, DISCARD, 2...
+            pages[i] = DISCARD
+
+    return [p for p in pages if p != DISCARD]
 
 
 def paginate_or_404(queryset, page, num_per_page=20):
@@ -592,9 +677,7 @@ def paginate_or_404(queryset, page, num_per_page=20):
     paginator = Paginator(queryset, num_per_page)
     try:
         page_obj = paginator.page(page)
-    except EmptyPage:
-        page_obj = paginator.page(paginator.num_pages)
-    except InvalidPage:
+    except EmptyPage, InvalidPage:
         raise Http404
     return paginator, page_obj
 
@@ -607,7 +690,8 @@ def location_question_results(
     question = get_object_or_404(Question.objects.select_related("survey"),
                                  pk=question_id,
                                  answer_is_public=True)
-    if not question.survey.can_have_public_submissions():
+    is_staff = request.user.is_staff
+    if not question.survey.can_have_public_submissions() and not is_staff:
         raise Http404
     featured = limit_results_to = False
     if survey_report_slug:
@@ -623,15 +707,17 @@ def location_question_results(
         for (option, icon) in icon_question.parsed_option_icon_pairs():
             if icon:
                 icon_by_answer[option] = icon
-        for answer in icon_question.answer_set.all().select_related("question"):
+        answer_set = icon_question.answer_set.all()
+        for answer in answer_set.select_related("question"):
             if answer.value in icon_by_answer:
                 icon = icon_by_answer[answer.value]
                 icon_lookup[answer.submission_id] = icon
 
     answers = question.answer_set.filter(
         ~Q(latitude=None),
-        ~Q(longitude=None),
-        submission__is_public=True).order_by("-submission__submitted_at")
+        ~Q(longitude=None)).order_by("-submission__submitted_at")
+    if not is_staff:
+        answers = answers.filter(submission__is_public=True)
     if featured:
         answers = answers.filter(submission__featured=True)
     answers = extra_from_filters(
@@ -641,7 +727,8 @@ def location_question_results(
         request.GET)
     limit_map_answers = int(limit_map_answers) if limit_map_answers else 0
     if limit_map_answers or limit_results_to:
-        answers = answers[:min(filter(None, [limit_map_answers, limit_results_to,]))]
+        answers = answers[:min(filter(None, [limit_map_answers,
+                                             limit_results_to,]))]
     entries = []
     view = "crowdsourcing.views.submission_for_map"
     for answer in answers:
@@ -657,8 +744,45 @@ def location_question_results(
     dump({"entries": entries}, response)
     return response
 
+def location_question_map(
+    request,
+    question_id,
+    display_id,
+    survey_report_slug=""):
+
+    question = Question.objects.get(pk=question_id)
+    if not question.answer_is_public and not request.user.is_staff:
+        raise Http404
+    report = None
+    limit = 0
+
+    if survey_report_slug:
+        report = SurveyReport.objects.get(slug=survey_report_slug,
+                                          survey=question.survey)
+        limit = report.limit_results_to
+    else:
+        report = _default_report(question.survey)
+
+    # This cast is not for validation since the urls file already guaranteed
+    # it would be a nonempty string of digits. It's simply because display_id
+    # is a string.
+    if int(display_id):
+        display = SurveyReportDisplay.objects.get(pk=display_id)
+    else:
+        for d in report.survey_report_displays:
+            if question.pk in [q.pk for q in d.questions()]:
+                display = d
+                display.limit_map_answers = limit
+
+    return render_to_response('crowdsourcing/location_question_map.html', dict(
+        display=display,
+        question=question,
+        report=report))
 
 def submission_for_map(request, id):
     template = 'crowdsourcing/submission_for_map.html'
-    sub = get_object_or_404(Submission.objects, is_public=True, pk=id)
+    if request.user.is_staff:
+        sub = get_object_or_404(Submission.objects, pk=id)
+    else:
+        sub = get_object_or_404(Submission.objects, is_public=True, pk=id)
     return render_to_response(template, dict(submission=sub), _rc(request))
